package view;

import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.net.URL;
import javax.swing.*;
import model.Direction;
import multithreading.Display;
import multithreading.ThreadedGameBoard;

public class GameWindow extends JFrame implements KeyListener, Display {
    final ThreadedGameBoard gameboard;

    /**
     * Creates new form GameWindow
     */
    public GameWindow() throws IOException {
        gameboard = new ThreadedGameBoard(25, this);

        initComponents();
        URL url = GameWindow.class.getClassLoader().getResource("res/pacman.png");
        setIconImage(Toolkit.getDefaultToolkit().getImage(url));
        setLocationRelativeTo(null);
        addKeyListener(this);
        
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent we) {
                gameboard.stop();
                System.exit(0);
            }
        });

        // Clients has to start AFTER the server is started, otherwise their
        // constructor will block on their InputStream
        gameboard.start();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        gamePanel1 = new GamePanel(gameboard);

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("PacMan");
        setResizable(false);

        gamePanel1.setBackground(new java.awt.Color(255, 255, 255));

        javax.swing.GroupLayout gamePanel1Layout = new javax.swing.GroupLayout(gamePanel1);
        gamePanel1.setLayout(gamePanel1Layout);
        gamePanel1Layout.setHorizontalGroup(
            gamePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 833, Short.MAX_VALUE)
        );
        gamePanel1Layout.setVerticalGroup(
            gamePanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 354, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(gamePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(gamePanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 354, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Windows".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(GameWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                try {
                    new GameWindow().setVisible(true);
                } catch (IOException ex) {}
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private view.GamePanel gamePanel1;
    // End of variables declaration//GEN-END:variables

    @Override
    public void keyTyped(KeyEvent ke) {
    }

    @Override
    public void keyPressed(KeyEvent ke) {
        String str = "";
        if (ke.getKeyCode() == KeyEvent.VK_A){
            gameboard.changePlayerDirection(0, Direction.LEFT);
        } else if (ke.getKeyCode() == KeyEvent.VK_D){
            gameboard.changePlayerDirection(0, Direction.RIGHT);
        } else if (ke.getKeyCode() == KeyEvent.VK_W){
            gameboard.changePlayerDirection(0, Direction.UP);
        } else if (ke.getKeyCode() == KeyEvent.VK_S){
            gameboard.changePlayerDirection(0, Direction.DOWN);
        } else if (ke.getKeyCode() == KeyEvent.VK_LEFT){
            gameboard.changePlayerDirection(1, Direction.LEFT);
        } else if (ke.getKeyCode() == KeyEvent.VK_RIGHT){
            gameboard.changePlayerDirection(1, Direction.RIGHT);
        } else if (ke.getKeyCode() == KeyEvent.VK_UP){
            gameboard.changePlayerDirection(1, Direction.UP);
        } else if (ke.getKeyCode() == KeyEvent.VK_DOWN){
            gameboard.changePlayerDirection(1, Direction.DOWN);
        } 
    }

    @Override
    public void keyReleased(KeyEvent ke) {
    }

    @Override
    public void refresh() {
        if (SwingUtilities.isEventDispatchThread()){
            gamePanel1.repaint();
        } else {
            SwingUtilities.invokeLater(() -> gamePanel1.repaint());
        }
    }

}
